@@ -1,5 +1,5 @@
 /******************************************************************************
- *  Wormux, a free clone of the game Worms from Team17.
+ *  Wormux is a convivial mass murder game.
  *  Copyright (C) 2001-2004 Lawrence Azzoug.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -16,9 +16,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  ******************************************************************************
- * Configuration de Wormux : toutes les variables qui sont intressantes 
- * modifier se retrouvent ici. Les variables ont une valeur par dfaut qui
- * peut tre modifie avec le fichier de configuration.
+ * Configuration of Wormux : store game config of every tunable variable of Wormux.
+ * Vars have a default value and can be change with the file configuration.
  *****************************************************************************/
 
 #ifndef CONFIG_H
@@ -27,7 +26,10 @@
 #include <list>
 #include <string>
 #include "../include/base.h"
+#include "../team/team_config.h"
 #include "../tool/xml_document.h"
+#include "../interface/keyboard.h"
+#include "../weapon/weapons_list.h"
 //-----------------------------------------------------------------------------
 #if defined(WIN32) || defined(__MINGW32__)
 #define PATH_SEPARATOR "\\"
@@ -46,10 +48,6 @@ public:
   static const int ALPHA = 0;
   static const int COLORKEY = 1;
 
-  // Divers
-  bool GetExterieurMondeVide() const;
-  void SetExterieurMondeVide(bool emv);
-
   bool GetDisplayEnergyCharacter() const;
   void SetDisplayEnergyCharacter(bool dec);
 
@@ -59,8 +57,19 @@ public:
   bool GetDisplayWindParticles() const;
   void SetDisplayWindParticles(bool dwp);
 
+  bool GetDefaultMouseCursor() const;
+  void SetDefaultMouseCursor(bool dmc);
+
+  bool GetScrollOnBorder() const;
+  void SetScrollOnBorder(bool sob);
+
+  bool IsNetworkActivated() const;
+
   int GetTransparency() const;
 
+  inline Keyboard * GetKeyboard() { return my_keyboard; }
+  inline WeaponsList * GetWeaponsList() { return my_weapons_list; }
+
   std::string GetTtfFilename() const;
 
   std::string GetDataDir() const;
@@ -69,6 +78,9 @@ public:
 
   // Tempory values (loaded from XML, but may change during running)
   struct tmp_xml_config{
+    struct tmp_xml_net{
+      bool enable_network;
+    } network;
     struct tmp_xml_screen{
       int width,height;
       bool fullscreen;
@@ -78,39 +90,42 @@ public:
       bool effects;
       uint frequency;
     } sound;
-    std::list<std::string> teams;
+    std::list<struct ConfigTeam> teams;
     std::string map_name;
   } tmp;
-  
+
   static Config * GetInstance();
-  void Init();
-  bool Load();
+// bool Load();
   void Apply();
   bool Save();
 
 protected:
-  bool ChargeVraiment();
-  bool ChargeXml (xmlpp::Element *xml);
+  bool LoadXml(xmlpp::Element *xml);
   void SetKeyboardConfig();
-  bool SauveXml();
-  std::string GetEnv(const std::string & name, const std::string & dft);
+  bool SaveXml();
+  std::string GetEnv(const std::string & name, const std::string &default_value);
 
   std::string m_game_mode;
-  bool m_xml_charge;
-  std::string m_nomfich;
+  bool m_xml_loaded;
+  std::string m_filename;
 
   std::string data_dir, locale_dir, personal_dir;
 
-  bool exterieur_monde_vide;
   bool display_energy_character;
   bool display_name_character;
   bool display_wind_particles;
+  bool default_mouse_cursor;
+  bool scroll_on_border;
   std::string ttf_filename;
 
   int transparency;
 
 private:
   Config();
+  // In french Clavier = keyboard
+  Keyboard * my_keyboard;
+  WeaponsList * my_weapons_list;
   static Config * singleton;
+  bool DoLoading(void);
 };
 #endif
