@@ -1,5 +1,5 @@
 /******************************************************************************
- *  Wormux, a free clone of the game Worms from Team17.
+ *  Wormux is a convivial mass murder game.
  *  Copyright (C) 2001-2004 Lawrence Azzoug.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
  ******************************************************************************
  * Sprite:     Simple sprite management
  ******************************************************************************
- * 2005/09/21: Jean-Christophe Duberga (jcduberga@gmx.de) 
+ * 2005/09/21: Jean-Christophe Duberga (jcduberga@gmx.de)
  *             Initial version
  *****************************************************************************/
 
@@ -62,7 +62,7 @@ Sprite::Sprite(const Sprite &other) :
   scale_x = other.scale_x;
   scale_y = other.scale_y;
   alpha = other.alpha;
-  rotation_deg = other.rotation_deg;
+  rotation_rad = other.rotation_rad;
   current_frame = other.current_frame;
   rot_hotspot = other.rot_hotspot;
   show = other.show;
@@ -77,7 +77,7 @@ void Sprite::Constructor() {
   frame_width_pix = frame_height_pix = 0;
   alpha = 1.0f;
   scale_x = scale_y = 1.0f;
-  rotation_deg = 0.0f;   
+  rotation_rad = 0.0f;
   SetRotation_HotSpot(center);
 }
 
@@ -88,12 +88,12 @@ void Sprite::Init(Surface& surface, const Point2i &frameSize, int nb_frames_x, i
    this->frame_height_pix = frameSize.y;
 
    surface.SetAlpha( 0, 0);
-   
+
    for( f.y = 0; f.y < nb_frames_y; f.y++)
      for( f.x = 0; f.x < nb_frames_x; f.x++){
        Surface new_surf = Surface(frameSize, SDL_SWSURFACE|SDL_SRCALPHA, true);
        Rectanglei sr(f * frameSize, frameSize);
-	  
+
        new_surf.Blit( surface, sr, Point2i(0, 0));
        frames.push_back( SpriteFrame(new_surf));
      }
@@ -114,17 +114,35 @@ void Sprite::SetSize(const Point2i &size){
 }
 
 unsigned int Sprite::GetWidth() const{
-   return (uint)((float)frame_width_pix * (scale_x>0?scale_x:-scale_x));
+   return static_cast<uint>(frame_width_pix * (scale_x>0?scale_x:-scale_x));
+}
+
+unsigned int Sprite::GetWidthMax() const{
+  if(!current_surface.IsNull() )
+    return current_surface.GetWidth();
+  else
+    return GetWidth();
 }
 
 unsigned int Sprite::GetHeight() const{
-   return (uint)((float)frame_height_pix * (scale_y>0?scale_y:-scale_y));
+   return static_cast<uint>(frame_height_pix * (scale_y>0?scale_y:-scale_y));
+}
+
+unsigned int Sprite::GetHeightMax() const{
+  if(!current_surface.IsNull() )
+    return current_surface.GetHeight();
+  else
+    return GetHeight();
 }
 
 Point2i Sprite::GetSize() const{
 	return Point2i(GetWidth(), GetHeight());
 }
 
+Point2i Sprite::GetSizeMax() const{
+	return Point2i(GetWidthMax(), GetHeightMax());
+}
+
 unsigned int Sprite::GetFrameCount(){
    return frames.size();
 }
@@ -193,40 +211,45 @@ float Sprite::GetAlpha(){
   return alpha;
 }
 
-void Sprite::SetRotation_deg( float angle_deg){
-   while(angle_deg >= 360.0)
-     angle_deg -= 360.0;
-   while(angle_deg < 0.0)
-     angle_deg += 360.0;
+void Sprite::SetRotation_rad( double angle_rad){
+   while(angle_rad > 2*M_PI)
+     angle_rad -= 2 * M_PI;
+   while(angle_rad <= -2*M_PI)
+     angle_rad += 2 * M_PI;
 
-   if(rotation_deg == angle_deg) return;
+   if(rotation_rad == angle_rad) return;
 
-   rotation_deg = angle_deg;
+   rotation_rad = angle_rad;
    cache.InvalidLastFrame();
 }
 
-float Sprite::GetRotation_deg()
+const double &Sprite::GetRotation_rad()
 {
-  return rotation_deg;
+  assert(rotation_rad > -2*M_PI && rotation_rad <= 2*M_PI);
+  return rotation_rad;
 }
 
 void Sprite::SetRotation_HotSpot( const Point2i new_hotspot)
 {
   rot_hotspot = user_defined;
   rhs_pos = new_hotspot;
+
+  if( rhs_pos.x * 2 == static_cast<int>(GetWidth()) &&
+      rhs_pos.y * 2 == static_cast<int>(GetHeight())  )
+    rot_hotspot = center; // avoid using Calculate_Rotation_Offset, thus avoiding a division by zero
 }
 
-void Sprite::Calculate_Rotation_Offset(int & rot_x, int & rot_y, Surface& tmp_surface){
+void Sprite::Calculate_Rotation_Offset(Surface& tmp_surface){
   const SpriteFrame& frame = GetCurrentFrameObject();
   const Surface &surface = frame.surface;
   // Calculate offset of the depending on hotspot rotation position :
-  
+
   int surfaceHeight = surface.GetHeight();
   int surfaceWidth = surface.GetWidth();
 
   //Do as if hotspot is center of picture:
-  rot_x = surfaceWidth  / 2 - tmp_surface.GetWidth()  / 2;
-  rot_y = surfaceHeight / 2 - tmp_surface.GetHeight() / 2;
+  rotation_point.x = surfaceWidth  / 2 - tmp_surface.GetWidth()  / 2;
+  rotation_point.y = surfaceHeight / 2 - tmp_surface.GetHeight() / 2;
 
   if(rot_hotspot == center)
       return;
@@ -250,30 +273,34 @@ void Sprite::Calculate_Rotation_Offset(int & rot_x, int & rot_y, Surface& tmp_su
   }
 
   Point2i rhs_pos_tmp;
-  rhs_pos_tmp.x = int((float)rhs_pos.x * scale_x);
-  rhs_pos_tmp.y = int((float)rhs_pos.y * scale_y);
-  surfaceWidth  = int((float)surfaceWidth  * scale_x);
-  surfaceHeight = int((float)surfaceHeight * scale_y);
+  rhs_pos_tmp.x = static_cast<uint>(rhs_pos.x * scale_x);
+  rhs_pos_tmp.y = static_cast<uint>(rhs_pos.y * scale_y);
+  surfaceWidth  = static_cast<uint>(surfaceWidth  * scale_x);
+  surfaceHeight = static_cast<uint>(surfaceHeight * scale_y);
 
   //Calculate the position of the hotspot after a rotation around the center of the surface:
   float rhs_dst; //Distance between center of the sprite and the hotspot
-  float rhs_angle; //Angle of the hotspot _before_ the rotation  
+  double rhs_angle; //Angle of the hotspot _before_ the rotation
 
   rhs_dst = sqrt(float((surfaceWidth /2 - rhs_pos_tmp.x)*(surfaceWidth /2 - rhs_pos_tmp.x)
                      + (surfaceHeight/2 - rhs_pos_tmp.y)*(surfaceHeight/2 - rhs_pos_tmp.y)));
-  rhs_angle = - acos ( float(rhs_pos_tmp.x - surfaceWidth/2) / rhs_dst );
+
+  if( rhs_dst == 0.0)
+    rhs_angle = 0.0;
+  else
+    rhs_angle = - acos ( float(rhs_pos_tmp.x - surfaceWidth/2) / rhs_dst );
+
   if(surfaceHeight/2 - rhs_pos.y < 0) rhs_angle = -rhs_angle;
-  float angle_rad = rotation_deg / 180.0 * M_PI; //Rotation angle of the sprite in radian
 
-  rhs_angle += angle_rad;
+  rhs_angle += rotation_rad;
 
-  Point2i rhs_new_pos =  Point2i(surfaceWidth /2 + int(cos(rhs_angle) * rhs_dst),
-                                 surfaceHeight/2 + int(sin(rhs_angle) * rhs_dst));
+  Point2i rhs_new_pos =  Point2i(surfaceWidth /2 + static_cast<uint>(cos(rhs_angle) * rhs_dst),
+                                 surfaceHeight/2 + static_cast<uint>(sin(rhs_angle) * rhs_dst));
 
-  rot_x -= rhs_new_pos.x;
-  rot_y -= rhs_new_pos.y;
-  rot_x += rhs_pos_tmp.x;
-  rot_y += rhs_pos_tmp.y;
+  rotation_point.x -= rhs_new_pos.x;
+  rotation_point.y -= rhs_new_pos.y;
+  rotation_point.x += rhs_pos_tmp.x;
+  rotation_point.y += rhs_pos_tmp.y;
 }
 
 void Sprite::Start(){
@@ -301,15 +328,9 @@ void Sprite::Blit( Surface &dest, int pos_x, int pos_y, int src_x, int src_y, ui
 
   RefreshSurface();
 
-   // Calculate offset of the sprite depending on hotspot rotation position :
-  int rot_x=0;
-  int rot_y=0;
-  if(rot_hotspot != center || rotation_deg!=0.0)
-    Calculate_Rotation_Offset(rot_x, rot_y, current_surface);
-
   Rectanglei srcRect (src_x, src_y, w, h);
-  Rectanglei dstRect (pos_x + rot_x, pos_y + rot_y, w, h);
-  
+  Rectanglei dstRect (pos_x + rotation_point.x, pos_y + rotation_point.y, w, h);
+
   if(alpha == 1.0)
     dest.Blit(current_surface, srcRect, dstRect.GetPosition());
   else
@@ -333,33 +354,37 @@ void Sprite::Finish(){
   {
   case SpriteAnimation::show_first_frame:
     current_frame = 0;
-    break;      
+    break;
   case SpriteAnimation::show_blank:
     show = false;
-    break;      
+    break;
   default:
   case SpriteAnimation::show_last_frame:
     current_frame = frames.size()-1;
-    break;      
+    break;
   }
   cache.InvalidLastFrame();
 }
 
 void Sprite::Update(){
   animation.Update();
-}    
+}
 
 void Sprite::Draw(const Point2i &pos){
-	if( !show )
-		return;
+  DrawXY(pos - camera.GetPosition());
+}
 
-	Blit(AppWormux::GetInstance()->video.window, pos - camera.GetPosition() );	
+void Sprite::DrawXY(const Point2i &pos){
+  if( !show )
+    return;
+
+  Blit(AppWormux::GetInstance()->video.window, pos);
 }
 
 void Sprite::Show() { show = true; }
 void Sprite::Hide() { show = false; }
 bool Sprite::IsFinished() const { return animation.IsFinished(); }
-  
+
 void Sprite::EnableRotationCache(unsigned int cache_size) {
   cache.EnableRotationCache(frames, cache_size);
 }
@@ -375,22 +400,22 @@ void Sprite::RefreshSurface()
   if(!cache.have_rotation_cache && !cache.have_flipping_cache)
   {
     if(!cache.have_lastframe_cache)
-      current_surface = frames[current_frame].surface.RotoZoom( -rotation_deg, scale_x, scale_y, SMOOTHING_OFF);
+      current_surface = frames[current_frame].surface.RotoZoom(-rotation_rad, scale_x, scale_y, SMOOTHING_OFF);
     else
     {
       if(cache.last_frame.IsNull() )
       {
 #ifdef BUGGY_SDLGFX
-        if(rotation_deg != 0.0 || (scale_x != 1.0 && scale_y == 1.0))
+        if(rotation_rad != 0.0 || (scale_x != 1.0 && scale_y == 1.0))
         {
-		  current_surface = frames[current_frame].surface.RotoZoom( -rotation_deg, scale_x, scale_y, SMOOTHING_OFF);
+          current_surface = frames[current_frame].surface.RotoZoom(-rotation_rad , scale_x, scale_y, SMOOTHING_OFF);
           cache.last_frame = current_surface;
         }
         else
         if(scale_x != 1.0 || scale_y != 1.0)
         {
 #endif
-		  current_surface = frames[current_frame].surface.RotoZoom( -rotation_deg, scale_x, scale_y, SMOOTHING_ON);
+          current_surface = frames[current_frame].surface.RotoZoom(-rotation_rad, scale_x, scale_y, SMOOTHING_ON);
           cache.last_frame = current_surface;
 #ifdef BUGGY_SDLGFX
         }
@@ -411,9 +436,9 @@ void Sprite::RefreshSurface()
   {
     if(cache.have_flipping_cache && !cache.have_rotation_cache)
     {
-      if(rotation_deg != 0.0 || scale_y != 1.0 || (scale_x != 1.0 && scale_x != -1.0))
+      if(rotation_rad != 0.0 || scale_y != 1.0 || (scale_x != 1.0 && scale_x != -1.0))
       {
-        current_surface = frames[current_frame].surface.RotoZoom( -rotation_deg, scale_x, scale_y, SMOOTHING_OFF );
+        current_surface = frames[current_frame].surface.RotoZoom( rotation_rad, scale_x, scale_y, SMOOTHING_OFF );
       }
       else
       if(scale_x == 1.0)
@@ -425,26 +450,32 @@ void Sprite::RefreshSurface()
     if(!cache.have_flipping_cache && cache.have_rotation_cache)
     {
       if(scale_x != 1.0 || scale_y != 1.0)
-        current_surface = frames[current_frame].surface.RotoZoom( -rotation_deg, scale_x, scale_y, SMOOTHING_OFF);
+        current_surface = frames[current_frame].surface.RotoZoom(rotation_rad, scale_x, scale_y, SMOOTHING_OFF);
       else
-        current_surface = cache.frames[current_frame].rotated_surface[(unsigned int)rotation_deg*cache.rotation_cache_size/360];
+        current_surface = cache.frames[current_frame].GetSurfaceForAngle(rotation_rad);
     }
     else
     {
       //cache.have_flipping_cache==true && cache.have_rotation_cache==true
       if((scale_x != 1.0 && scale_x != -1.0)  || scale_y != 1.0)
-        current_surface = frames[current_frame].surface.RotoZoom( -rotation_deg, scale_x, scale_y, SMOOTHING_OFF);
+        current_surface = frames[current_frame].surface.RotoZoom( rotation_rad, scale_x, scale_y, SMOOTHING_OFF);
       else
       {
         //Scale_y == 1.0
         if(scale_x == 1.0)
-          current_surface = cache.frames[current_frame].rotated_surface[(unsigned int)rotation_deg*cache.rotation_cache_size/360];
+          current_surface = cache.frames[current_frame].GetSurfaceForAngle(rotation_rad);
         else
-          current_surface = cache.frames[current_frame].rotated_flipped_surface[(unsigned int)rotation_deg*cache.rotation_cache_size/360];
+          current_surface = cache.frames[current_frame].GetFlippedSurfaceForAngle(rotation_rad);
       }
     }
   }
   assert( !current_surface.IsNull() );
+
+  // Calculate offset of the sprite depending on hotspot rotation position :
+  rotation_point.x=0;
+  rotation_point.y=0;
+  if(rot_hotspot != center || rotation_rad!=0.0)
+    Calculate_Rotation_Offset(current_surface);
 }
 
 Surface Sprite::GetSurface() {
