@@ -1,5 +1,5 @@
 /******************************************************************************
- *  Wormux, a free clone of the game Worms from Team17.
+ *  Wormux is a convivial mass murder game.
  *  Copyright (C) 2001-2004 Lawrence Azzoug.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -18,7 +18,7 @@
  ******************************************************************************
  * Sprite:     Simple sprite management
  ******************************************************************************
- * 2005/09/21: Jean-Christophe Duberga (jcduberga@gmx.de) 
+ * 2005/09/21: Jean-Christophe Duberga (jcduberga@gmx.de)
  *             Initial version
  *****************************************************************************/
 
@@ -33,9 +33,9 @@
 #include "include/base.h"
 
 typedef enum {
-  top_left, 
-  top_center, 
-  top_right, 
+  top_left,
+  top_center,
+  top_right,
   left_center,
   center,
   right_center,
@@ -50,7 +50,7 @@ class Sprite
 public:
   SpriteCache cache;
   SpriteAnimation animation;
-	
+
 public:
   explicit Sprite();
   explicit Sprite( Surface surface);
@@ -58,16 +58,22 @@ public:
 
   void Init(Surface& surface, const Point2i &frameSize, int nb_frames_x, int nb_frames_y);
   Surface GetSurface();
-   
+
   // Frame number
   unsigned int GetCurrentFrame() const;
-  void SetCurrentFrame( unsigned int frame_no);    
+  void SetCurrentFrame( unsigned int frame_no);
   unsigned int GetFrameCount();
 
   // Size
   unsigned int GetWidth() const;
+  unsigned int GetWidthMax() const; // gives height of the surface
+                                     // (takes rotations into acount)
   unsigned int GetHeight() const;
+  unsigned int GetHeightMax() const; // gives height of the surface
+                                     // (takes rotations into acount)
   Point2i GetSize() const;
+  Point2i GetSizeMax() const;
+
   void GetScaleFactors( float &scale_x, float &scale_y);
   void SetSize(unsigned int w, unsigned int h);
   void SetSize(const Point2i &size);
@@ -76,11 +82,12 @@ public:
   void ScaleSize(Point2i size);
 
   // Rotation
-  void SetRotation_deg( float angle_deg);
-  float GetRotation_deg();
+  void SetRotation_rad( double angle_rad);
+  const double &GetRotation_rad();
   void SetRotation_HotSpot( const Point2i new_hotspot);
   void SetRotation_HotSpot( const Rotation_HotSpot rhs) { rot_hotspot = rhs; };
-  
+  const Point2i& GetRotationPoint() { return rotation_point; };
+
   SpriteFrame& operator[] (unsigned int frame_no);
   const SpriteFrame& operator[] (unsigned int frame_no) const;
   const SpriteFrame& GetCurrentFrameObject() const;
@@ -106,34 +113,36 @@ public:
   // Show flag
   void Show();
   void Hide();
-     
+
   // Draw
   void Blit(Surface &dest, uint pox_x, uint pos_y);
   void Blit(Surface &dest, const Point2i &pos);
   void Blit(Surface &dest, const Rectanglei &srcRect, const Point2i &destPos);
   void Blit(Surface &dest, int pox_x, int pos_y, int src_x, int src_y, uint w, uint h);
   void Draw(const Point2i &pos);
+  void DrawXY(const Point2i &pos);
+
+   void RefreshSurface();
 
 private:
    Surface current_surface;
    bool show;
-
    // Frames
    unsigned int current_frame;
    int frame_width_pix,frame_height_pix;
    std::vector<SpriteFrame> frames;
-   
+
    // Gfx
    float alpha;
    float scale_x,scale_y;
-   float rotation_deg;
+   double rotation_rad;
    Point2i rhs_pos;
    Rotation_HotSpot rot_hotspot;
+   Point2i rotation_point;
 
 private:
    void Constructor();
-   void RefreshSurface();
-   void Calculate_Rotation_Offset(int & rot_x, int & rot_y, Surface& tmp_surface);
+   void Calculate_Rotation_Offset(Surface& tmp_surface);
 };
 
 #endif /* _SPRITE_H */
