@@ -1,5 +1,5 @@
 /******************************************************************************
- *  Wormux, a free clone of the game Worms from Team17.
+ *  Wormux is a convivial mass murder game.
  *  Copyright (C) 2001-2004 Lawrence Azzoug.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -24,70 +24,96 @@
 #include "../graphic/colors.h"
 #include "../include/app.h"
 
-Box::Box(const Rectanglei &rect, bool _visible) : Widget( rect ){
+Box::Box(const Rectanglei &rect, bool _visible) : WidgetList( rect )
+{
   visible = _visible;
   margin = 5;
   border.SetValues(5, 5);
 }
 
-Box::~Box(){
-  std::list<Widget *>::iterator it;
-  for (it = widgets.begin();
-      it != widgets.end();
-      it++){
-    delete *it;
-    *it = NULL;
-  }
-  widgets.clear();
+Box::~Box()
+{
 }
 
-void Box::Draw(const Point2i &mousePosition){
+void Box::Redraw(const Rectanglei& rect,
+		 Surface& surf)
+{
+  // Redraw bottom layer container
+  WidgetList::Redraw(rect, surf);
+
+  if ( visible ) {
+    // Redraw
+    surf.BoxColor(rect, defaultOptionColorBox);
+  }
+}
+
+void Box::Update(const Point2i &mousePosition,
+		 const Point2i &lastMousePosition,
+		 Surface& surf)
+{
+  if (need_redrawing) {
+    Draw(mousePosition, surf);
+  }
+
+  WidgetList::Update(mousePosition, surf);
+  need_redrawing = false;
+}
+
+void Box::Draw(const Point2i &mousePosition,
+	       Surface& surf) const
+{
   Rectanglei rect(position, size);
 	
   if( visible ){
-    AppWormux::GetInstance()->video.window.BoxColor(rect, defaultOptionColorBox);
-    AppWormux::GetInstance()->video.window.RectangleColor(rect, defaultOptionColorRect);
-  }
-
-  std::list<Widget *>::iterator it;
-  for (it = widgets.begin(); 
-       it != widgets.end(); 
-       ++it){
-    (*it)->Draw(mousePosition);
+    surf.BoxColor(rect, defaultOptionColorBox);
+    surf.RectangleColor(rect, defaultOptionColorRect,2);
   }
 }
 
-bool Box::Clic (const Point2i &mousePosition, uint button){
-  std::list<Widget *>::iterator it;
-  for (it = widgets.begin(); 
-       it != widgets.end(); 
-       ++it){
-    if( (*it)->Clic(mousePosition, button) )
-      return true;
-  }
-
-  return false;
+Widget* Box::Clic (const Point2i &mousePosition, uint button)
+{
+  return WidgetList::Clic(mousePosition, button);
 }
 
-void Box::SetMargin (uint _margin){
+void Box::SetMargin (uint _margin)
+{
   margin = _margin;
 }
 
-void Box::SetBorder (const Point2i &newBorder){
-	border = newBorder;
+void Box::SetBorder (const Point2i &newBorder)
+{
+  border = newBorder;
 }
 
-VBox::VBox(const Rectanglei &rect, bool _visible) : Box(rect, _visible){
-	size.y = 1;
+// --------------------------------------------------
+
+VBox::VBox(const Rectanglei &rect, bool _visible) : Box(rect, _visible)
+{
+  size.y = 1;
 }
 
-void VBox::AddWidget(Widget * a_widget){
+void VBox::DelFirstWidget()
+{
+  int w_height = widget_list.front()->GetSizeY();
+  WidgetList::DelFirstWidget();
+  //Make all remaining widget go up:
+  for( std::list<Widget*>::iterator it = widget_list.begin(); 
+       it != widget_list.end(); 
+       ++it )
+  {
+    (*it)->SetPositionY((*it)->GetPositionY() - w_height - margin);
+  }
+  size.y -= w_height + margin;
+}
+
+void VBox::AddWidget(Widget * a_widget)
+{
   assert(a_widget != NULL);
 
   uint _y;
 
-  if(!widgets.empty())
-    _y = widgets.back()->GetPositionY() + widgets.back()->GetSizeY();
+  if(!widget_list.empty())
+    _y = widget_list.back()->GetPositionY() + widget_list.back()->GetSizeY();
   else
     _y = position.y + border.y - margin;
 
@@ -96,20 +122,20 @@ void VBox::AddWidget(Widget * a_widget){
 			    size.x - 2 * border.x,
 			    a_widget->GetSizeY() ));
 
-  widgets.push_back(a_widget);
-
   size.y = a_widget->GetPositionY() + a_widget->GetSizeY() - position.y + border.y;
+  WidgetList::AddWidget(a_widget);
 }
 
-void VBox::SetSizePosition(const Rectanglei &rect){
+void VBox::SetSizePosition(const Rectanglei &rect)
+{
   position = rect.GetPosition();
   int _y = rect.GetPositionY();
   std::list<Widget *>::iterator it;
-  for( it = widgets.begin(); 
-       it != widgets.end(); 
+  for( it = widget_list.begin(); 
+       it != widget_list.end(); 
        ++it ){
 
-    if( it == widgets.begin() )
+    if( it == widget_list.begin() )
       _y += border.y - margin;
 
     (*it)->SetSizePosition( Rectanglei(position.x + border.x,
@@ -120,18 +146,21 @@ void VBox::SetSizePosition(const Rectanglei &rect){
   }
 }
 
-HBox::HBox(const Rectanglei &rect, bool _visible) :
-  Box(rect, _visible){
-	  size.x = 1;
+// --------------------------------------------------
+
+HBox::HBox(const Rectanglei &rect, bool _visible) : Box(rect, _visible)
+{
+  size.x = 1;
 }
 
-void HBox::AddWidget(Widget * a_widget){
+void HBox::AddWidget(Widget * a_widget)
+{
   assert(a_widget != NULL);
 
   uint _x;
 
-  if (!widgets.empty())
-    _x = widgets.back()->GetPositionX() + widgets.back()->GetSizeX();
+  if (!widget_list.empty())
+    _x = widget_list.back()->GetPositionX() + widget_list.back()->GetSizeX();
   else 
     _x = position.x + border.x - margin;
 
@@ -140,21 +169,22 @@ void HBox::AddWidget(Widget * a_widget){
 			    a_widget->GetSizeX(), 
 			    size.y - 2 * border.y) );
 
-  widgets.push_back(a_widget);
-
   size.x = a_widget->GetPositionX() + a_widget->GetSizeX() - position.x + border.x;
+
+  WidgetList::AddWidget(a_widget);
 }
 
-void HBox::SetSizePosition(const Rectanglei &rect){
+void HBox::SetSizePosition(const Rectanglei &rect)
+{
   position = rect.GetPosition();
   int _x = rect.GetPositionX();
 	
   std::list<Widget *>::iterator it;
-  for( it = widgets.begin(); 
-       it != widgets.end(); 
+  for( it = widget_list.begin(); 
+       it != widget_list.end(); 
        ++it ){
 
-    if( it == widgets.begin() )
+    if( it == widget_list.begin() )
       _x += border.x - margin;
 
     (*it)->SetSizePosition( Rectanglei(_x + margin,
