@@ -1,5 +1,5 @@
 /******************************************************************************
- *  Wormux, a free clone of the game Worms from Team17.
+ *  Wormux is a convivial mass murder game.
  *  Copyright (C) 2001-2004 Lawrence Azzoug.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -27,7 +27,7 @@ NullWidget::NullWidget (const Rectanglei &rect) : Widget(rect){
 NullWidget::~NullWidget(){
 }
 
-void NullWidget::Draw(const Point2i &mousePosition){
+void NullWidget::Draw(const Point2i &mousePosition, Surface& surf) const{
 }
 
 void NullWidget::SetSizePosition(const Rectanglei &rect){
