@@ -1,5 +1,5 @@
 /******************************************************************************
- *  Wormux, a free clone of the game Worms from Team17.
+ *  Wormux is a convivial mass murder game.
  *  Copyright (C) 2001-2004 Lawrence Azzoug.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -29,7 +29,7 @@ class NullWidget : public Widget
  public:
   NullWidget(const Rectanglei &rect);
   ~NullWidget();
-  void Draw(const Point2i &mousePosition);
+  void Draw(const Point2i &mousePosition, Surface& surf) const;
   void SetSizePosition(const Rectanglei &rect);
 };
 
