@@ -1,5 +1,5 @@
 /******************************************************************************
- *  Wormux, a free clone of the game Worms from Team17.
+ *  Wormux is a convivial mass murder game.
  *  Copyright (C) 2001-2004 Lawrence Azzoug.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -24,6 +24,8 @@
 #define QUESTION_H
 //-----------------------------------------------------------------------------
 #include "../include/base.h"
+#include "../graphic/sprite.h"
+#include "../graphic/text.h"
 #include <string>
 #include <list>
 #include <SDL_events.h>
@@ -32,23 +34,24 @@
 
 class Question
 {
-public:
-  // Message to display
-  std::string message;
+  Sprite* background;
 
   // A choice = a key return a value
-  typedef struct choix_t
+  class choice_t
   {
-    int m_key;
-    int m_val;
-    choix_t (int key, int value) 
-    { m_key = key; m_val = value; }
-  } choice_t;
+    private:
+      int m_key;
+      int m_val;
+    public:
+      choice_t (int key, int value)
+      { m_key = key; m_val = value; };
+      inline const int & key() const { return m_key; };
+      inline const int & val() const { return m_val; };
+  };
 
   // Choices list
   std::list<choice_t> choices;
-  typedef std::list<choix_t>::iterator choice_iterator;
-
+  typedef std::list<choice_t>::iterator choice_iterator;
   // Default choice used when another key is pressed
   struct s_default_choice
   {
@@ -56,16 +59,27 @@ public:
     int value;
   } default_choice;
 
- private:
   int TreatsKey (SDL_Event &event);
-  void Draw();
 
- public:
+  // Message to display
+  Text * text;
+
+public:
   Question();
-  void Set(const std::string &message, 
-	   bool default_active, 
-	   int default_value);
-  int AskQuestion ();
+  ~Question();
+
+  void Set(const std::string &message,
+	   bool default_active,
+	   int default_value,
+      const std::string &bg_sprite="");
+  int Ask();
+  void Draw() const;
+
+  inline void add_choice(int key, int value)
+  {
+    return this->choices.push_back(choice_t(key,value));
+  }
+
 };
 
 //-----------------------------------------------------------------------------
