@@ -1,5 +1,5 @@
 /******************************************************************************
- *  Wormux, a free clone of the game Worms from Team17.
+ *  Wormux is a convivial mass murder game.
  *  Copyright (C) 2001-2004 Lawrence Azzoug.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -21,41 +21,35 @@
 
 #include "constant.h"
 
-const std::string Constants::VERSION = "0.7.4";
+const std::string Constants::VERSION = PACKAGE_VERSION;
 
-const std::string Constants::DEFAULT_DATADIR = INSTALL_DATADIR;
-const std::string Constants::DEFAULT_LOCALEDIR = INSTALL_LOCALEDIR;
-const std::string Constants::DEFAULT_FONT_PATH = FONT_FILE;
 const std::string Constants::ENV_DATADIR = "WORMUX_DATADIR";
 const std::string Constants::ENV_LOCALEDIR = "WORMUX_LOCALEDIR";
 const std::string Constants::ENV_FONT_PATH = "WORMUX_FONT_PATH";
 
-// Nombre de boucles maximum pour le test ...
-const uint Constants::NBR_BCL_MAX_EST_VIDE = 200; // Un objet est dans le vide ?
-
 std::vector<std::string> Constants::AUTHORS;
 
 const std::string Constants::WEB_SITE = "http://www.wormux.org";
-const std::string Constants::EMAIL = "wormux-dev@gna.org";
+const std::string Constants::EMAIL = "wormux-dev .AT. gna .DOT. org";
 
-// Dimensions min/max du terrain (en pixel)
+// Size min/max of the map (pixels)
 const Point2i Constants::MAP_MIN_SIZE = Point2i(100, 200);
 const int Constants::MAP_MAX_SIZE = 4000*4000;
 
-// Hauteur minimale libre pour que le terrain soit qualifi " d'ouvert "
-const uint Constants::HAUT_MIN_TERRAIN_OUVERT = 40;
-
 Constants * Constants::singleton = NULL;
 
 Constants::Constants()
 {
-  AUTHORS.push_back ("Anthony CARRE");
-  AUTHORS.push_back ("Jean-Christophe DUBERGA");
-  AUTHORS.push_back ("Laurent DEFERT SIMONNEAU");
   AUTHORS.push_back ("Lawrence AZZOUG");
+  AUTHORS.push_back ("Anthony CARRE");
+  AUTHORS.push_back ("Laurent DEFERT SIMONNEAU");
+  AUTHORS.push_back ("Jean-Christophe DUBERGA");
   AUTHORS.push_back ("Matthieu FERTRE");
-  AUTHORS.push_back ("Olivier Boyer");
+  AUTHORS.push_back ("Sebastien GONZALVE");
+  AUTHORS.push_back ("Reiner HERRMANN");
   AUTHORS.push_back ("Renaud LOTTIAUX");
+  AUTHORS.push_back ("Yannig PERRE");
+  AUTHORS.push_back ("Olivie SERRES");
   AUTHORS.push_back ("Victor STINNER");
 }
 
