@@ -1,5 +1,5 @@
 /******************************************************************************
- *  Wormux, a free clone of the game Worms from Team17.
+ *  Wormux is a convivial mass murder game.
  *  Copyright (C) 2001-2004 Lawrence Azzoug.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  ******************************************************************************
- * Liste des terrains.
+ * Maps list
  *****************************************************************************/
 
 #include "maps_list.h"
@@ -31,26 +31,24 @@
 #include <sys/stat.h>
 #endif
 
-ListeTerrain lst_terrain;
-
-InfoTerrain::InfoTerrain ()
-{ 
-  m_donnees_chargees = false;
+InfoMap::InfoMap ()
+{
+  is_data_loaded = false;
   nb_mine = 0;
+  nb_barrel = 0;
   wind.nb_sprite = 0;
   wind.need_flip = false;
-  infinite_bg = false;
 }
 
-bool InfoTerrain::Init (const std::string &map_name, 
-			            const std::string &directory)
+bool InfoMap::Init (const std::string &map_name,
+		    const std::string &directory)
 {
   std::string nomfich;
 
   m_directory = directory;
 
   res_profile = NULL;
-  m_donnees_chargees = false;
+  is_data_loaded = false;
 
   try
   {
@@ -59,13 +57,13 @@ bool InfoTerrain::Init (const std::string &map_name,
     // Load resources
     if (!IsFileExist(nomfich))
       return false;
-    res_profile = resource_manager.LoadXMLProfile( nomfich, true), 
+    res_profile = resource_manager.LoadXMLProfile( nomfich, true),
     // Load preview
     preview = resource_manager.LoadImage( res_profile, "preview");
     // Load other informations
-    LitDocXml doc;
-    if (!doc.Charge (nomfich)) return false;
-    if (!TraiteXml (doc.racine())) return false;
+    XmlReader doc;
+    if (!doc.Load(nomfich)) return false;
+    if (!ProcessXmlData(doc.GetRoot())) return false;
   }
   catch (const xmlpp::exception &e)
   {
@@ -75,16 +73,16 @@ bool InfoTerrain::Init (const std::string &map_name,
               << e.what() << std::endl;
     return false;
   }
- 
+
   MSG_DEBUG("map.load", "Map loaded: %s", map_name.c_str());
 
   return true;
 }
 
-bool InfoTerrain::TraiteXml (xmlpp::Element *xml)
+bool InfoMap::ProcessXmlData(xmlpp::Element *xml)
 {
   // Read author informations
-  xmlpp::Element *author = LitDocXml::AccesBalise (xml, "author");
+  xmlpp::Element *author = XmlReader::GetMarker(xml, "author");
   if (author != NULL) {
     std::string
       a_name,
@@ -92,45 +90,39 @@ bool InfoTerrain::TraiteXml (xmlpp::Element *xml)
       a_country,
       a_email;
 
-    LitDocXml::LitString (author, "name", a_name);
-    LitDocXml::LitString (author, "nickname", a_nickname);
-    if (!LitDocXml::LitString (author, "country", a_country))
+    XmlReader::ReadString(author, "name", a_name);
+    XmlReader::ReadString(author, "nickname", a_nickname);
+    if (!XmlReader::ReadString(author, "country", a_country))
       a_country = "?";
-    if (!LitDocXml::LitString (author, "email", a_email))
+    if (!XmlReader::ReadString(author, "email", a_email))
       a_email = "?";
 
-    if (!a_nickname.empty()) 
+    if (!a_nickname.empty())
       author_info = Format
-	(_("%s <%s> aka %s from %s"),
-	 a_name.c_str(),
-	 a_email.c_str(),
-	 a_nickname.c_str(),
-	 a_country.c_str());
+          (_("%s <%s> aka %s from %s"),
+           a_name.c_str(),
+           a_email.c_str(),
+           a_nickname.c_str(),
+           a_country.c_str());
     else
       author_info = Format
-	(_("%s <%s> from %s"),
-	 a_name.c_str(),
-	 a_email.c_str(),
-	 a_country.c_str());
+          (_("%s <%s> from %s"),
+           a_name.c_str(),
+           a_email.c_str(),
+           a_country.c_str());
   }
 
-  LitDocXml::LitString (xml, "name", name);
-  LitDocXml::LitBool (xml, "water", use_water);
-  LitDocXml::LitUint (xml, "nb_mine", nb_mine);
-  LitDocXml::LitBool (xml, "is_open", is_opened);
-//  LitDocXml::LitBool (xml, "infinite_background", infinite_bg);
+  XmlReader::ReadString(xml, "name", name);
+  XmlReader::ReadBool(xml, "water", use_water);
+  XmlReader::ReadUint(xml, "nb_mine", nb_mine);
+  XmlReader::ReadUint(xml, "nb_barrel", nb_barrel);
+  XmlReader::ReadBool(xml, "is_open", is_opened);
 
-  if(!is_opened && infinite_bg)
-  {
-    std::cout << _("<infinite_background> needs <is_open> to be set to work correctly!!") << std::endl;
-    infinite_bg = false;
-  }
-
-  xmlpp::Element *xmlwind = LitDocXml::AccesBalise (xml, "wind");
+  xmlpp::Element *xmlwind = XmlReader::GetMarker(xml, "wind");
   if (xmlwind != NULL)
   {
-    LitDocXml::LitUint (xmlwind, "nbr_sprite", wind.nb_sprite);
-    LitDocXml::LitBool (xmlwind, "need_flip", wind.need_flip);
+    XmlReader::ReadUint(xmlwind, "nbr_sprite", wind.nb_sprite);
+    XmlReader::ReadBool(xmlwind, "need_flip", wind.need_flip);
 
     if (wind.nb_sprite > MAX_WIND_OBJECTS)
       wind.nb_sprite = MAX_WIND_OBJECTS ;
@@ -142,69 +134,54 @@ bool InfoTerrain::TraiteXml (xmlpp::Element *xml)
   return true;
 }
 
-void InfoTerrain::LoadData(){
-  if (m_donnees_chargees)
+void InfoMap::LoadData()
+{
+  if (is_data_loaded)
     return;
-  m_donnees_chargees = true;
- 
+  is_data_loaded = true;
+
   MSG_DEBUG("map.load", "Map data loaded: %s", name.c_str());
 
-  img_terrain = resource_manager.LoadImage(res_profile, "map");
-  img_ciel = resource_manager.LoadImage(res_profile,"sky");
+  img_ground = resource_manager.LoadImage(res_profile, "map");
+  img_sky = resource_manager.LoadImage(res_profile,"sky");
 }
 
-void InfoTerrain::FreeData(){
-  img_ciel.Free();
-  img_terrain.Free();
-  m_donnees_chargees = false;
-}
-
-Surface InfoTerrain::LitImgTerrain(){ 
-  LoadData(); 
-  return img_terrain;
-}
-
-Surface InfoTerrain::LitImgCiel(){ 
-  LoadData(); 
-  return img_ciel;
-}
-
-bool InfoTerrain::DonneesChargees() const{ 
-   return m_donnees_chargees; 
-}
-
-ListeTerrain::ListeTerrain(){
-  m_init = false;
-  terrain_actif = -1;
-}
-
-void ListeTerrain::LoadOneMap (const std::string &dir, const std::string &file){
-  std::string fullname = dir+file;
-
-#if !defined(WIN32) || defined(__MINGW32__)
-  struct stat stat_file;
-  if (file[0] == '.') return;
-  if (stat(fullname.c_str(), &stat_file) != 0) return;
-  if (!S_ISDIR(stat_file.st_mode)) return;
-#endif
-	
-  InfoTerrain nv_terrain;  
-  bool ok = nv_terrain.Init (file, fullname + PATH_SEPARATOR);
-  if (!ok) return;
-
-  std::cout << (liste.empty()?" ":", ") << file;
-  std::cout.flush();
-  liste.push_back(nv_terrain);
-}
-
-void ListeTerrain::Init()
+void InfoMap::FreeData()
 {
-  if (m_init) return;
-  
-  m_init = true;
-  liste.clear() ;
+  img_sky.Free();
+  img_ground.Free();
+  is_data_loaded = false;
+}
+
+Surface InfoMap::ReadImgGround()
+{
+  LoadData();
+  return img_ground;
+}
+
+Surface InfoMap::ReadImgSky()
+{
+  LoadData();
+  return img_sky;
+}
+
+MapsList* MapsList::singleton = NULL;
+
+MapsList* MapsList::GetInstance()
+{
+  if (singleton == NULL) {
+    singleton = new MapsList();
+  }
+
+  return singleton;
+}
+
+MapsList::MapsList()
+{
+  lst.clear() ;
 
   std::cout << "o " << _("Load maps:");
+  terrain_actif = -1;
 
   Config * config = Config::GetInstance();
   std::string dirname = config->GetDataDir() + PATH_SEPARATOR + "map" + PATH_SEPARATOR;
@@ -243,7 +220,7 @@ void ListeTerrain::Init()
   dirname = config->GetPersonalDir() + PATH_SEPARATOR + "map";
   dir = opendir(dirname.c_str());
   if (dir != NULL) {
-    while ((file = readdir(dir)) != NULL) 
+    while ((file = readdir(dir)) != NULL)
       LoadOneMap (dirname, file->d_name);
     closedir (dir);
   }
@@ -251,52 +228,82 @@ void ListeTerrain::Init()
   std::cout << std::endl << std::endl;
 
   // On a au moins une carte ?
-  if (liste.size() < 1)
+  if (lst.size() < 1)
     Error(_("You need at least one valid map !"));
 
-  std::sort(lst_terrain.liste.begin(), lst_terrain.liste.end(), compareMaps);
+  std::sort(lst.begin(), lst.end(), compareMaps);
 }
 
-int ListeTerrain::FindMapById (const std::string &id)
+void MapsList::LoadOneMap (const std::string &dir, const std::string &file)
+{
+  std::string fullname = dir+file;
+
+#if !defined(WIN32) || defined(__MINGW32__)
+  struct stat stat_file;
+  if (file[0] == '.') return;
+  if (stat(fullname.c_str(), &stat_file) != 0) return;
+  if (!S_ISDIR(stat_file.st_mode)) return;
+#endif
+
+  InfoMap nv_terrain;
+  bool ok = nv_terrain.Init (file, fullname + PATH_SEPARATOR);
+  if (!ok) return;
+
+  std::cout << (lst.empty()?" ":", ") << file;
+  std::cout.flush();
+  lst.push_back(nv_terrain);
+}
+
+int MapsList::FindMapById (const std::string &id)
 {
   iterator
-    terrain=liste.begin(),
-    fin_terrain=liste.end();
+    terrain=lst.begin(),
+    fin_terrain=lst.end();
   uint i=0;
-  for (; i < liste.size(); ++i)
-    if (liste[i].name == id)
+  for (; i < lst.size(); ++i)
+    if (lst[i].ReadName() == id)
       return i;
   return -1;
 }
 
-void ListeTerrain::ChangeTerrainNom (const std::string &nom){
+void MapsList::SelectMapByName (const std::string &nom)
+{
   int index = FindMapById (nom);
 
   if (index == -1){
     index = 0;
     std::cout << Format(_("! Map %s not found :-("), nom.c_str()) << std::endl;
   }
-  ChangeTerrain (index);
+  SelectMapByIndex (index);
 }
 
-void ListeTerrain::ChangeTerrain (uint index){
-  assert (index < liste.size());
+void MapsList::SelectMapByIndex (uint index)
+{
+  assert (index < lst.size());
   if (terrain_actif == (int)index)
     return;
 
   terrain_actif = index;
 }
 
-InfoTerrain& ListeTerrain::TerrainActif(){
+int MapsList::GetActiveMapIndex ()
+{
+  return terrain_actif;
+}
+
+InfoMap& MapsList::ActiveMap()
+{
   assert (0 <= terrain_actif);
-  return liste.at(terrain_actif);
+  return lst.at(terrain_actif);
 }
 
-InfoTerrain& TerrainActif(){
-  return lst_terrain.TerrainActif();
+InfoMap& ActiveMap()
+{
+  return MapsList::GetInstance()->ActiveMap();
 }
 
-bool compareMaps(const InfoTerrain& a, const InfoTerrain& b){  
-  return a.name < b.name;
+bool compareMaps(const InfoMap& a, const InfoMap& b)
+{
+  return a.ReadName() < b.ReadName();
 }
 
