@@ -1,5 +1,5 @@
 /******************************************************************************
- *  Wormux, a free clone of the game Worms from Team17.
+ *  Wormux is a convivial mass murder game.
  *  Copyright (C) 2001-2004 Lawrence Azzoug.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -22,7 +22,11 @@
 
 #include "graphic/surface.h"
 
-const Point2i CELL_SIZE(128,128);
+const Point2i CELL_SIZE(64, 64);
+
+#ifdef DEBUG
+//#define DBG_TILE
+#endif
 
 class TileItem
 {
@@ -34,9 +38,13 @@ public:
   virtual unsigned char GetAlpha(const Point2i &pos) = 0;
   virtual void Dig(const Point2i &position, const Surface& dig) = 0;
   virtual void Dig(const Point2i &center, const uint radius) = 0;
+  virtual void MergeSprite(const Point2i &position, Surface& spr) {};
   virtual Surface GetSurface() = 0;
-  virtual void SyncBuffer() = 0; // (if needed)
-  virtual void Draw(const Point2i &pos);
+  virtual void Draw(const Point2i &pos) = 0;
+  virtual bool IsTotallyEmpty() const = 0;
+#ifdef DBG_TILE
+  virtual void FillWithRGB(Uint8 r, Uint8 g, Uint8 b) {};
+#endif
 };
 
 class TileItem_Empty : public TileItem
@@ -49,21 +57,32 @@ public:
   void Dig(const Point2i &position, const Surface& dig){};
   Surface GetSurface(){return *new Surface();};
   void Dig(const Point2i &center, const uint radius) {};
-  void SyncBuffer(){};
-  void Draw(const Point2i &pos){};
+  void Draw(const Point2i &pos);
+  bool IsTotallyEmpty() const {return true;};
 };
 
 class TileItem_AlphaSoftware : public TileItem
 {
+  unsigned char* last_filled_pixel;
+
 public:
-  TileItem_AlphaSoftware(const Point2i &size);
+  bool need_check_empty;
+  bool need_delete;
+
+TileItem_AlphaSoftware(const Point2i &size);
   ~TileItem_AlphaSoftware();
 
   unsigned char GetAlpha(const Point2i &pos);
-  Surface GetSurface();
   void Dig(const Point2i &position, const Surface& dig);
   void Dig(const Point2i &center, const uint radius);
-  void SyncBuffer();
+  void MergeSprite(const Point2i &position, Surface& spr);
+  void Draw(const Point2i &pos);
+
+  bool NeedDelete() const {return need_delete; };
+  void CheckEmpty();
+  void ResetEmptyCheck();
+
+  bool IsTotallyEmpty() const {return false;};
 
 private:
   TileItem_AlphaSoftware(const TileItem_AlphaSoftware &copy);
@@ -71,12 +90,17 @@ private:
   unsigned char GetAlpha_Index0(const Point2i &pos);
   inline unsigned char GetAlpha_Index3(const Point2i &pos);
   inline unsigned char GetAlpha_Generic(const Point2i &pos);
+  Surface GetSurface();
 
   void Empty(const int start_x, const int end_x, unsigned char* buf, const int bpp);
   void Darken(const int start_x, const int end_x, unsigned char* buf, const int bpp);
 
   Point2i m_size;
   Surface m_surface;
+
+#ifdef DBG_TILE
+  void FillWithRGB(Uint8 r, Uint8 g, Uint8 b);
+#endif
 };
 
 #endif
