@@ -1,5 +1,5 @@
 /******************************************************************************
- *  Wormux, a free clone of the game Worms from Team17.
+ *  Wormux is a convivial mass murder game.
  *  Copyright (C) 2001-2004 Lawrence Azzoug.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -35,9 +35,16 @@ class WindParticle : public PhysicalObj
 {
 public:
   Sprite *sprite;
-     
+
 public:
   WindParticle(std::string& xml_file);
+  inline WindParticle(const WindParticle & aparticle):
+    PhysicalObj(aparticle)
+    {
+      assert(aparticle.sprite);
+      sprite = new Sprite(*aparticle.sprite);
+    };
+  ~WindParticle() { delete sprite; };
   void Resize(double size);
   void Draw();
   void Refresh();
@@ -45,7 +52,6 @@ public:
 
 class Wind
 {
-  BarreProg barre;
   long m_val, m_nv_val;
   uint m_last_move;
   uint m_last_part_mvt;
@@ -61,7 +67,6 @@ public:
   void SetVal (long val);
   void Refresh();
   void Reset();
-  void Draw();
   void DrawParticles();
   void RandomizeParticlesPos(); // Put particles randomly on the screen
 };
