@@ -1,5 +1,5 @@
 /******************************************************************************
- *  Wormux, a free clone of the game Worms from Team17.
+ *  Wormux is a convivial mass murder game.
  *  Copyright (C) 2001-2004 Lawrence Azzoug.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -21,109 +21,116 @@
 
 #include "../object/objects_list.h"
 //-----------------------------------------------------------------------------
+#include "barrel.h"
 #include "../include/app.h"
 #include "../game/game_loop.h"
 #include "../map/map.h"
 #include "../map/maps_list.h"
 #include "../map/camera.h"
+#include "../tool/debug.h"
 #include "../tool/random.h"
 #include "../tool/rectangle.h"
+#include "../game/time.h"
 #include "../weapon/mine.h"
 #include <vector>
+#include <iostream>
 
 //-----------------------------------------------------------------------------
 ObjectsList lst_objects;
 //-----------------------------------------------------------------------------
 
 // Initialise la liste des objets standards
-void ObjectsList::Init()
+void ObjectsList::PlaceMines()
 {
-  lst.clear();
-
-  for (uint i=0; i<lst_terrain.TerrainActif().nb_mine; ++i)
+  MSG_DEBUG("lst_objects","Placing mines");
+  for (uint i=0; i<ActiveMap().GetNbMine(); ++i)
   {
     ObjMine *obj = new ObjMine(*MineConfig::GetInstance());
 
-    if (obj->PutRandomly(false, MineConfig::GetInstance()->detection_range * 40 *1.5 ))
-      // 40 is current PIXEL_PER_METER and detection range is in meter
-      AddObject (obj);
+    if (obj->PutRandomly(false, MineConfig::GetInstance()->detection_range * PIXEL_PER_METER *1.5 ))
+      // detection range is in meter
+      push_back(obj);
     else
       delete obj;
   }
 }
 
-//-----------------------------------------------------------------------------
-
-void ObjectsList::AddObject (PhysicalObj* obj)
+void ObjectsList::PlaceBarrels()
 {
-  lst.push_back (object_t(obj,false));
+  MSG_DEBUG("lst_objects","Placing barrels");
+  for (uint i= 0; i<ActiveMap().GetNbBarrel(); ++i)
+  {
+    PetrolBarrel *obj = new PetrolBarrel();
+
+    if (obj->PutRandomly(false, 20.0))
+      push_back(obj);
+    else
+      delete obj;
+  }
 }
 
+
 ObjectsList::~ObjectsList()
 {
-
-  std::list<object_t>::iterator object;
-  for (object = lst.begin();
-       object != lst.end();
-       ++object) {
-    if((*object).ptr)
-      delete (*object).ptr;
-  }
+  FreeMem();
 }
 
 //-----------------------------------------------------------------------------
-
-void ObjectsList::RemoveObject (PhysicalObj* obj)
-{
-  FOR_EACH_OBJECT(it)
-  {
-    if ( it->ptr == obj)
-    {
-      it->to_remove = true;
-      // please, do not call camera.StopFollowingObj(obj) here,
-      // because we want to see the end of an explosion when removing projectiles
-      return;
-    }
-  }
-}
-
-//-----------------------------------------------------------------------------
-
 void ObjectsList::Refresh()
 {
-  ObjectsList::iterator object=lst_objects.Begin();
+  ObjectsList::iterator object=begin();
 
-  while(object != lst_objects.End())
+  while(object != end())
   {
-    if (!object->to_remove && !object->ptr->IsGhost()) {
-      object->ptr->UpdatePosition();
-      object->ptr->Refresh();
+    (*object)->UpdatePosition();
+    (*object)->Refresh();
+    if((*object)->IsGhost())
+      object = lst_objects.erase(object);
+    else
       object++;
-    } else {
-      object = lst.erase(object);
-    }
   }
 }
 
 //-----------------------------------------------------------------------------
-
 void ObjectsList::Draw()
 {
-  FOR_EACH_OBJECT(object) object->ptr->Draw ();
+  for (ObjectsList::iterator it = begin();
+       it != end();
+       ++it)
+  {
+    assert((*it) != NULL);
+
+    if (!(*it)->IsGhost())
+      (*it)->Draw();
+  }
 }
 
-
 //-----------------------------------------------------------------------------
-
-// Tous les objets sont prts ? (ou alors un objet est en cours
-// d'animation ?)
 bool ObjectsList::AllReady()
 {
   FOR_EACH_OBJECT(object)
   {
-    if (!object->ptr->IsReady()) return false;
+    if (!(*object)->IsImmobile())
+    {
+      MSG_DEBUG("lst_objects", "\"%s\" is not ready ( IsImmobile()==fasle )", (*object)->GetName().c_str());
+      return false;
+    }
   }
   return true;
 }
 
 //-----------------------------------------------------------------------------
+
+void ObjectsList::FreeMem()
+{
+  MSG_DEBUG("lst_objects", "Erasing object list");
+  ObjectsList::iterator object;
+  for (object = begin();
+       object != end();
+       ++object) {
+    if((*object))
+      delete (*object);
+  }
+  clear();
+}
+
