@@ -1,5 +1,5 @@
 /******************************************************************************
- *  Wormux, a free clone of the game Worms from Team17.
+ *  Wormux is a convivial mass murder game.
  *  Copyright (C) 2001-2004 Lawrence Azzoug.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -16,71 +16,56 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  ******************************************************************************
- * Liste des objets qui sont sur le plateau de jeu.
+ * Stores most of the object playing in the game
  *****************************************************************************/
 
-#ifndef OBJETS_H
-#define OBJETS_H
+#ifndef OBJECTS_LIST_H
+#define OBJECTS_LIST_H
 //-----------------------------------------------------------------------------
 #include "../include/base.h"
 #include "../object/physical_obj.h"
 #include <list>
 //-----------------------------------------------------------------------------
 
-// Boucle pour chaque objet de la liste des objets (non fantome)
+// Loop for all objects
 #define FOR_ALL_OBJECTS(object) \
-  for (ObjectsList::iterator object=lst_objects.Begin(), \
-       end=lst_objects.End(); \
+  for (ObjectsList::iterator object=lst_objects.begin(), \
+       end=lst_objects.end(); \
        object != end; \
-       ++object) \
-	if(!object->to_remove)
+       ++object)
 
 //-----------------------------------------------------------------------------
 
-// Boucle pour chaque objet de la liste des objets (non fantome)
+// Loop for all objects that aren't out of the screen
 #define FOR_EACH_OBJECT(object) \
   FOR_ALL_OBJECTS(object) \
-  if (!object->ptr->IsGhost())
+  if (!(*object)->IsGhost())
 
 //-----------------------------------------------------------------------------
 
-class ObjectsList
+class ObjectsList : public std::list<PhysicalObj*>
 {
-public:
-  typedef struct object_t {
-    PhysicalObj* ptr;
-    bool to_remove;
-    object_t(PhysicalObj* o, bool e) { ptr = o; to_remove = e; }
-  } object_t;
+  public:
+    inline void RemoveObject(PhysicalObj * obj) { remove(obj);};
+    typedef std::list<PhysicalObj*>::iterator iterator;
 
-  typedef std::list<object_t>::iterator iterator;
+  public:
+    ~ObjectsList();
+    inline void AddObject(PhysicalObj * obj) { push_back(obj);};
 
-private:
-  std::list<object_t> lst;
+  // Call the Refresh method of all the objects
+    void Refresh();
+  // Call the Draw method of all the objects
+    void Draw();
 
-public:
-  void Init();
-  ~ObjectsList();
+    bool AllReady();
 
-  // Ajoute un objet  la liste
-  void AddObject (PhysicalObj* obj);
+  // Place mines randomly on the map
+    void PlaceMines();
+  // Place barrels randomly on the map
+    void PlaceBarrels();
 
-  // Retire un objet de la liste
-  void RemoveObject (PhysicalObj* obj);
-
-  // Refresh des tous les objets
-  void Refresh();
-
-  // Draw tous les objets
-  void Draw();
-
-  // Tous les objets sont prts ? (ou alors un objet est en cours
-  // d'animation ?)
-  bool AllReady();
-
-  // Dbut/fin de la liste
-  iterator Begin() { return lst.begin(); }
-  iterator End() { return lst.end(); }
+    void FreeMem();
 };
 
 extern ObjectsList lst_objects;
