@@ -1,5 +1,5 @@
 /******************************************************************************
- *  Wormux, a free clone of the game Worms from Team17.
+ *  Wormux is a convivial mass murder game.
  *  Copyright (C) 2001-2004 Lawrence Azzoug.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -26,6 +26,7 @@
 
 #include "../object/physics.h"
 #include <stdlib.h>
+#include <iostream>
 #include "../game/config.h"
 #include "../game/game_mode.h"
 #include "../game/time.h"
@@ -35,8 +36,8 @@
 
 // Physical constants
 const double STOP_REBOUND_LIMIT = 0.5 ;
-const double AIR_RESISTANCE_FACTOR = 14 ;
-const double PHYS_DELTA_T = 0.01 ;         // Physical simulation time step
+const double AIR_RESISTANCE_FACTOR = 40.0 ;
+const double PHYS_DELTA_T = 0.02 ;         // Physical simulation time step
 const double PENDULUM_REBOUND_FACTOR = 0.8 ;
 
 Physics::Physics ()
@@ -49,7 +50,7 @@ Physics::Physics ()
   m_fix_point_dxy.Clear();
   m_rope_angle.Clear();
   m_rope_length.Clear();
-  
+
   m_rope_elasticity = 10.0 ;
   m_elasticity_damping = 0.20 ;
   m_balancing_damping = 0.40 ;
@@ -84,14 +85,14 @@ void Physics::SetPhysXY(const Point2d &position){
 	SetPhysXY(position.x, position.y);
 }
 
-double Physics::GetPhysX() const 
+double Physics::GetPhysX() const
 {
-  return m_pos_x.x0; 
+  return m_pos_x.x0;
 }
 
-double Physics::GetPhysY() const 
-{ 
-  return m_pos_y.x0; 
+double Physics::GetPhysY() const
+{
+  return m_pos_y.x0;
 }
 
 Point2d Physics::GetPos() const{
@@ -112,19 +113,19 @@ void Physics::SetMass(double mass)
 
 // Set the wind factor
 void Physics::SetWindFactor (double factor)
-{ 
+{
   m_wind_factor = factor;
 }
 
 // Set the air resist factor
 void Physics::SetAirResistFactor (double factor)
-{ 
+{
   m_air_resist_factor = factor;
 }
 
 // Set the wind factor
 void Physics::SetGravityFactor (double factor)
-{ 
+{
   m_gravity_factor = factor;
 }
 
@@ -166,7 +167,7 @@ void Physics::AddSpeedXY (Point2d vector)
   if (!was_moving && IsMoving()) StartMoving();
 }
 
-void Physics::GetSpeed(double &norm, double &angle)
+void Physics::GetSpeed(double &norm, double &angle) const
 {
   Point2d speed ;
 
@@ -174,29 +175,35 @@ void Physics::GetSpeed(double &norm, double &angle)
     case FreeFall:
       GetSpeedXY(speed);
       norm = speed.Norm();
-      angle = GetSpeedAngle();
+      angle = speed.ComputeAngle();
       break ;
 
     case Pendulum:
       // Transform angular speed to linear speed.
-      
+
       norm = fabs(m_rope_length.x0 * m_rope_angle.x1);
-      
+
       if (m_rope_angle.x1 > 0)
 	angle = fabs(m_rope_angle.x0) ;
       else
 	angle = fabs(m_rope_angle.x0) - M_PI ;
-      
+
       if (m_rope_angle.x0 < 0)
 	angle = -angle ;
       break;
 
+    case NoMotion:
+      norm = 0.0;
+      angle = 0.0;
+      break;
+
     default:
+      assert(false);
       break ;
   }
 }
 
-void Physics::GetSpeedXY(Point2d &vector)
+void Physics::GetSpeedXY(Point2d &vector) const
 {
   if(!IsMoving())
     {
@@ -206,12 +213,19 @@ void Physics::GetSpeedXY(Point2d &vector)
   vector.SetValues(m_pos_x.x1, m_pos_y.x1);
 }
 
-double Physics::GetAngularSpeed()
+Point2d Physics::GetSpeed() const
+{
+  Point2d tmp;
+  GetSpeedXY(tmp);
+  return tmp;
+}
+
+double Physics::GetAngularSpeed() const
 {
   return m_rope_angle.x1 ;
 }
 
-double Physics::GetSpeedAngle()
+double Physics::GetSpeedAngle() const
 {
   double angle ;
   Point2d speed ;
@@ -272,7 +286,7 @@ void Physics::SetPhysFixationPointXY(double g_x, double g_y, double dx,
       // Compute the initial angle
       V.x = fix_point_x - g_x ;
       V.y = fix_point_y - g_y ;
-      m_rope_angle.x0 = M_PI/2 - V.ComputeAngle() ;
+      m_rope_angle.x0 = M_PI_2 - V.ComputeAngle() ;
 
       // Convert the linear speed to angular speed.
       m_rope_angle.x1 = (m_pos_x.x1 * cos(m_rope_angle.x0) +
@@ -312,7 +326,7 @@ void Physics::ChangePhysRopeSize(double dl)
     return ;
 
   bool was_moving = IsMoving();
-  
+
   m_rope_length.x0 += dl ;
 
   // Recompute angular speed depending on the new rope length.
@@ -340,16 +354,15 @@ void Physics::StartMoving()
   m_last_move = Time::GetInstance()->Read();
 
   if (m_motion_type == NoMotion)
-    m_motion_type = FreeFall ;  
+    m_motion_type = FreeFall ;
 
   MSG_DEBUG ("physic.physic", "Start moving.");
 }
 
 void Physics::StopMoving()
 {
-  if (IsMoving())
-	  MSG_DEBUG ("physic.physic", "End of a mouvement...");
-
+  if (IsMoving()) MSG_DEBUG ("physic.physic", "End of a movement...");
+  // Always called by PhysicalObj::StopMoving
   m_pos_x.x1 = 0 ;
   m_pos_x.x2 = 0 ;
   m_pos_y.x1 = 0 ;
@@ -375,21 +388,6 @@ bool Physics::IsFalling() const
 	   ( m_pos_y.x1 > 0.1) );
 }
 
-double Physics::GetContactSurface(double angle)
-{
-  double x_surface, y_surface ;
-
-  // Very approximative contact surface
-
-  x_surface = m_phys_width * sin(angle) ;
-  x_surface *= x_surface ;
-		  
-  y_surface = m_phys_height * cos(angle) ;
-  y_surface *= y_surface ;
-
-  return 0.5 * (x_surface + y_surface) ;
-}
-
 // Compute the next position of the object during a pendulum motion.
 void Physics::ComputePendulumNextXY (double delta_t)
 {
@@ -417,7 +415,7 @@ void Physics::ComputePendulumNextXY (double delta_t)
 		              +m_extern_force.x / m_rope_length.x0 * cos (m_rope_angle.x0),
                       delta_t);
 
-  double x = m_fix_point_gnd.x - m_fix_point_dxy.x 
+  double x = m_fix_point_gnd.x - m_fix_point_dxy.x
              + m_rope_length.x0 * sin(m_rope_angle.x0);
   double y = m_fix_point_gnd.y - m_fix_point_dxy.y
              + m_rope_length.x0 * cos(m_rope_angle.x0);
@@ -442,7 +440,7 @@ void Physics::ComputeFallNextXY (double delta_t)
 
   // Weight force = m * g
 
-  //  printf ("av : (%5f,%5f) - (%5f,%5f) - (%5f,%5f)\n", m_pos_x.x0,
+  // printf ("av : (%5f,%5f) - (%5f,%5f) - (%5f,%5f)\n", m_pos_x.x0,
   //	  m_pos_y.x0, m_pos_x.x1, m_pos_y.x1, m_pos_x.x2, m_pos_y.x2);
 
   weight_force = GameMode::GetInstance()->gravity * m_gravity_factor * m_mass ;
@@ -451,11 +449,11 @@ void Physics::ComputeFallNextXY (double delta_t)
 
   wind_force = wind.GetStrength() * m_wind_factor ;
 
-  // Air resistanceance factor = K * contact_surface
+  // Air resistanceance factor
 
   GetSpeed(speed_norm, speed_angle);
 
-  air_resistance_factor = AIR_RESISTANCE_FACTOR * GetContactSurface(speed_angle) * m_air_resist_factor ;
+  air_resistance_factor = AIR_RESISTANCE_FACTOR * m_air_resist_factor ;
 
   MSG_DEBUG( "physic.fall", "Fall %s; mass %5f, weight %5f, wind %5f, air %5f", typeid(*this).name(), m_mass, weight_force,wind_force, air_resistance_factor);
 
@@ -467,9 +465,10 @@ void Physics::ComputeFallNextXY (double delta_t)
   m_pos_y.ComputeOneEulerStep(m_mass, air_resistance_factor, 0,
 		      weight_force + m_extern_force.y, delta_t);
 
-  //  printf ("F : Pd(%5f) EF(%5f)\n", weight_force, m_extern_force.y);
 
-  //  printf ("ap : (%5f,%5f) - (%5f,%5f) - (%5f,%5f)\n", m_pos_x.x0,
+    // printf ("F : Pd(%5f) EF(%5f)\n", weight_force, m_extern_force.y);
+
+   // printf ("ap : (%5f,%5f) - (%5f,%5f) - (%5f,%5f)\n", m_pos_x.x0,
   //	  m_pos_y.x0, m_pos_x.x1, m_pos_y.x1, m_pos_x.x2, m_pos_y.x2);
 }
 
@@ -491,9 +490,6 @@ void Physics::RunPhysicalEngine()
   double step_t, delta_t = (Time::GetInstance()->Read() - m_last_move) / 1000.0;
   Point2d oldPos;
   Point2d newPos;
-  Point2d contactPos;
-  double contact_angle;
-  bool contact = false;
 
   step_t = PHYS_DELTA_T;
 
@@ -503,30 +499,26 @@ void Physics::RunPhysicalEngine()
   // Compute object move for each physical engine time step.
 
   while (delta_t > 0.0){
-      if (delta_t < PHYS_DELTA_T)
-        step_t = delta_t ;
+    if (delta_t < PHYS_DELTA_T)
+      step_t = delta_t ;
 
-      oldPos = GetPos();
-      
-      newPos = ComputeNextXY(step_t);
+    oldPos = GetPos();
 
-      if( !(newPos == oldPos) )  {
-	// The object has moved. Notify the son class.
-	MSG_DEBUG( "physic.move", "Move %s (%f, %f) -> (%f, %f)", typeid(*this).name(), oldPos.x, oldPos.y, newPos.x, newPos.y);
-		contact = NotifyMove(oldPos, newPos, contactPos, contact_angle);
-      }
+    newPos = ComputeNextXY(step_t);
 
-      if (contact){
-		  MSG_DEBUG( "physic.coll", "Collision durant le dplacement (%f, %f) -> (%f, %f)", oldPos.x, oldPos.y, newPos.x, newPos.y);
-		  Rebound(contactPos, contact_angle);
-      }
+    if( newPos != oldPos)  {
+      // The object has moved. Notify the son class.
+      MSG_DEBUG( "physic.move", "Move %s (%f, %f) -> (%f, %f)", typeid(*this).name(), oldPos.x, oldPos.y, newPos.x, newPos.y);
+      NotifyMove(oldPos, newPos);
+    }
 
-      delta_t -= PHYS_DELTA_T ;
+    delta_t -= PHYS_DELTA_T ;
   }
 
   return;
 }
 
+/* contact_angle is the angle of the surface we are rebounding on */
 void Physics::Rebound(Point2d contactPos, double contact_angle)
 {
   double norme, angle;
@@ -535,58 +527,59 @@ void Physics::Rebound(Point2d contactPos, double contact_angle)
   GetSpeed(norme, angle);
 
   switch (m_motion_type) {
-    case FreeFall :
-		if (m_rebounding){ 
-			// Compute rebound angle.
-			if(contact_angle == -1.0)
-				angle = angle + M_PI ;
-			else
-				angle -= 2.0 * (angle - contact_angle);
+  case FreeFall :
+    if (m_rebounding)
+    {
+      // Compute rebound angle.
+      /* if no tangent rebound in the opposit direction */
+      if(contact_angle == NAN)
+	angle = angle + M_PI ;
+      else
+	angle =  M_PI - angle -2.0 *  contact_angle;
 
-			// Apply rebound factor to the object speed.
-			norme = norme * m_rebound_factor;
-	  	  
-			// Apply the new speed to the object.
-			SetSpeed(norme, angle);
-	  
-			// Check if we should stop rebounding.
-			if (norme < STOP_REBOUND_LIMIT){
-				StopMoving();
-				return;
-		 	}
-			SignalRebound();
-		}else{
-			SignalFallEnding();
-			StopMoving();
-		}
-		break;
+      // Apply rebound factor to the object speed.
+      norme = norme * m_rebound_factor;
 
-    case Pendulum:
-		{
-			Point2d V ;
+      // Apply the new speed to the object.
+      SetSpeed(norme, angle);
 
-			// Recompute new angle.
-			V.x = m_pos_x.x0 + m_fix_point_dxy.x - m_fix_point_gnd.x;
-			V.y = m_pos_y.x0 + m_fix_point_dxy.y - m_fix_point_gnd.y;
-		
-			m_rope_angle.x0 = M_PI_2 - V.ComputeAngle();
-      
-			// Convert the linear speed of the rebound to angular speed.
-			V.x = PENDULUM_REBOUND_FACTOR * norme * cos(angle);
-			V.y = PENDULUM_REBOUND_FACTOR * norme * sin(angle);
+      // Check if we should stop rebounding.
+      if (norme < STOP_REBOUND_LIMIT){
+	StopMoving();
+	return;
+      }
+      SignalRebound();
+    }
+    else
+      StopMoving();
+    break;
 
-			angle = angle + M_PI;
+  case Pendulum:
+    {
+      Point2d V ;
 
-			m_rope_angle.x1 = (norme * cos(angle) * cos(m_rope_angle.x0) +
-				norme * sin(angle) * sin(m_rope_angle.x0) ) / m_rope_length.x0;
+      // Recompute new angle.
+      V.x = m_pos_x.x0 + m_fix_point_dxy.x - m_fix_point_gnd.x;
+      V.y = m_pos_y.x0 + m_fix_point_dxy.y - m_fix_point_gnd.y;
 
-			m_rope_angle.x2 = 0;
-			m_extern_force.Clear();
-		}
-		break ;
-     
-    default:
-      break ;
+      m_rope_angle.x0 = M_PI_2 - V.ComputeAngle();
+
+      // Convert the linear speed of the rebound to angular speed.
+      V.x = PENDULUM_REBOUND_FACTOR * norme * cos(angle);
+      V.y = PENDULUM_REBOUND_FACTOR * norme * sin(angle);
+
+      angle = angle + M_PI;
+
+      m_rope_angle.x1 = (norme * cos(angle) * cos(m_rope_angle.x0) +
+			 norme * sin(angle) * sin(m_rope_angle.x0) ) / m_rope_length.x0;
+
+      m_rope_angle.x2 = 0;
+      m_extern_force.Clear();
+    }
+    break ;
+
+  default:
+    break ;
   }
 
 }
@@ -594,6 +587,4 @@ void Physics::Rebound(Point2d contactPos, double contact_angle)
 void Physics::SignalGhostState(bool)  {}
 void Physics::SignalDeath() {}
 void Physics::SignalDrowning() {}
-void Physics::SignalFallEnding() {}
 void Physics::SignalRebound() {}
-
