@@ -1,5 +1,5 @@
 /******************************************************************************
- *  Wormux, a free clone of the game Worms from Team17.
+ *  Wormux is a convivial mass murder game.
  *  Copyright (C) 2001-2004 Lawrence Azzoug.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -30,7 +30,7 @@
 #define _(X) gettext(X)
 
 std::string Format (const char *format, ...);
-void InitI18N();
+void InitI18N(const std::string &dir);
 void I18N_SetDir(const std::string &dir);
 
 #endif
