@@ -1,5 +1,5 @@
 /******************************************************************************
- *  Wormux, a free clone of the game Worms from Team17.
+ *  Wormux is a convivial mass murder game.
  *  Copyright (C) 2001-2004 Lawrence Azzoug.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  ******************************************************************************
- * Statistics. 
+ * Statistics.
  *****************************************************************************/
 
 #ifndef STATS_H
@@ -26,7 +26,7 @@
 #include <string>
 
 #ifdef DEBUG
-#  define ENABLE_STATS
+//#  define ENABLE_STATS
 #endif
 
 #ifdef ENABLE_STATS
