@@ -1,5 +1,5 @@
 /******************************************************************************
- *  Wormux, a free clone of the game Worms from Team17.
+ *  Wormux is a convivial mass murder game.
  *  Copyright (C) 2001-2004 Lawrence Azzoug.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -20,36 +20,35 @@
  *****************************************************************************/
 
 #include "jetpack.h"
-#include "weapon_tools.h"
+#include "explosion.h"
 #include "../game/game.h"
 #include "../game/game_loop.h"
 #include "../game/game_mode.h"
 #include "../game/time.h"
 #include "../interface/game_msg.h"
 #include "../map/camera.h"
+#include "../network/network.h"
 #include "../object/physical_obj.h"
 #include "../sound/jukebox.h"
 #include "../team/teams_list.h"
 #include "../tool/i18n.h"
+#include "../character/move.h"
+#include "../include/action_handler.h"
 
-
-// Espace entre l'espace en l'image
-const uint ESPACE = 5;
-const double JETPACK_FORCE = 750.0;
+const double JETPACK_FORCE = 2000.0;
 
 const uint DELTA_FUEL_DOWN = 200 ;  // Delta time between 2 fuel unit consumption.
 
 JetPack::JetPack() : Weapon(WEAPON_JETPACK, "jetpack",
-			    new WeaponConfig(),
-			    NEVER_VISIBLE)
+                            new WeaponConfig(),
+                            NEVER_VISIBLE)
 {
-  m_name = _("JetPack");
+  m_name = _("Jetpack");
   m_unit_visibility = VISIBLE_ONLY_WHEN_ACTIVE;
 
   override_keys = true ;
-  use_unit_on_first_shoot = false;  
+  use_unit_on_first_shoot = false;
 
-  m_name = _("jetpack");
   m_x_force = 0.0;
   m_y_force = 0.0;
   channel = -1;
@@ -57,43 +56,50 @@ JetPack::JetPack() : Weapon(WEAPON_JETPACK, "jetpack",
 
 void JetPack::Refresh()
 {
+  if(!ActiveTeam().IsLocal())
+    return;
+
   Point2d F;
 
   if (m_is_active)
+  {
+    F.x = m_x_force ;
+    F.y = m_y_force ;
+
+    ActiveCharacter().SetExternForceXY(F);
+    ActiveCharacter().UpdatePosition();
+    SendCharacterPosition();
+    Action a(Action::ACTION_SET_CHARACTER_PHYSICS);
+    a.StoreActiveCharacter();
+    network.SendAction(&a);
+
+    if( !F.IsNull() )
     {
-      F.x = m_x_force ;
-      F.y = m_y_force ;
+      // We are using fuel !!!
+      uint current = Time::GetInstance()->Read() ;
+      double delta = (double)(current - m_last_fuel_down);
 
-      ActiveCharacter().SetExternForceXY(F);
-      ActiveCharacter().UpdatePosition();
-
-      if( !F.IsNull() )
-	{
-	  // We are using fuel !!!
-
-	  uint current = Time::GetInstance()->Read() ;
-	  double delta = (double)(current - m_last_fuel_down);
-
-	  while (delta >= DELTA_FUEL_DOWN)
-	    {
-	      if (EnoughAmmoUnit())
-		{
-		  UseAmmoUnit();
-		  m_last_fuel_down += DELTA_FUEL_DOWN ;
-		  delta -= DELTA_FUEL_DOWN ;
-		}
-	      else
-		{
-		  p_Deselect();
-		  break;
-		}
-	    }
-	}
+      while (delta >= DELTA_FUEL_DOWN)
+      {
+        if (EnoughAmmoUnit())
+        {
+          UseAmmoUnit();
+          m_last_fuel_down += DELTA_FUEL_DOWN ;
+          delta -= DELTA_FUEL_DOWN ;
+        }
+        else
+        {
+          p_Deselect();
+          break;
+        }
+      }
     }
+  }
 }
 
 void JetPack::p_Select()
 {
+  ActiveCharacter().SetClothe("jetpack");
 }
 
 void JetPack::p_Deselect()
@@ -103,30 +109,34 @@ void JetPack::p_Deselect()
   m_y_force = 0;
   ActiveCharacter().SetExternForce(0,0);
   StopUse();
-  ActiveCharacter().SetSkin("walking");
+  camera.SetCloseFollowing(false);
+  ActiveCharacter().SetClothe("normal");
+  ActiveCharacter().SetMovement("walk");
 }
 
 void JetPack::StartUse()
 {
+  ActiveCharacter().SetMovement("jetpack-fire");
   if ( (m_x_force == 0) && (m_y_force == 0))
     {
       m_last_fuel_down = Time::GetInstance()->Read();
       channel = jukebox.Play(ActiveTeam().GetSoundProfile(),"weapon/jetpack", -1);
 
-      camera.ChangeObjSuivi (&ActiveCharacter(),true, true, true); 
-// 			     bool suit, bool recentre,
-// 			     bool force_recentrage=false);
+      camera.FollowObject (&ActiveCharacter(),true, true, true);
+      camera.SetCloseFollowing(true);
+//                           bool suit, bool recentre,
+//                           bool force_recentrage=false);
     }
 }
 
 void JetPack::StopUse()
 {
+  ActiveCharacter().SetMovement("jetpack-nofire");
   if (m_x_force == 0.0 && m_y_force == 0.0)
   {
     if(channel != -1)
       jukebox.Stop(channel);
     channel = -1;
-    ActiveCharacter().SetSkin("jetpack");
   }
 }
 
@@ -134,25 +144,22 @@ void JetPack::GoUp()
 {
   StartUse();
   m_y_force = -(ActiveCharacter().GetMass() * GameMode::GetInstance()->gravity + JETPACK_FORCE) ;
-  ActiveCharacter().SetSkin("jetpack-up");
 }
 
 void JetPack::GoLeft()
 {
   StartUse();
-  ActiveCharacter().SetSkin("jetpack-left-right");
-  ActiveCharacter().image->Scale (-1, 1);
-
   m_x_force = - JETPACK_FORCE ;
+  if(ActiveCharacter().GetDirection() == Body::DIRECTION_RIGHT)
+    ActiveCharacter().SetDirection(Body::DIRECTION_LEFT);
 }
 
 void JetPack::GoRight()
 {
   StartUse();
-  ActiveCharacter().SetSkin("jetpack-left-right");
-  ActiveCharacter().image->Scale (1, 1);
-
   m_x_force = JETPACK_FORCE ;
+  if(ActiveCharacter().GetDirection() == Body::DIRECTION_LEFT)
+    ActiveCharacter().SetDirection(Body::DIRECTION_RIGHT);
 }
 
 void JetPack::StopUp()
@@ -173,36 +180,33 @@ void JetPack::StopRight()
   StopUse();
 }
 
-void JetPack::HandleKeyEvent(int action, int event_type)
+void JetPack::HandleKeyEvent(Action::Action_t action, Keyboard::Key_Event_t event_type)
 {
   switch (action) {
-    case ACTION_UP:
-      if (event_type == KEY_PRESSED)
-	GoUp();
-      else
-	if (event_type == KEY_RELEASED)
-	  StopUp();
+    case Action::ACTION_UP:
+      if (event_type == Keyboard::KEY_PRESSED)
+        GoUp();
+      else if (event_type == Keyboard::KEY_RELEASED)
+        StopUp();
       break ;
 
-    case ACTION_MOVE_LEFT:
-      if (event_type == KEY_PRESSED)
-	GoLeft();
-      else
-	if (event_type == KEY_RELEASED)
-	  StopLeft();
+    case Action::ACTION_MOVE_LEFT:
+      if (event_type == Keyboard::KEY_PRESSED)
+        GoLeft();
+      else if (event_type == Keyboard::KEY_RELEASED)
+        StopLeft();
       break ;
 
-    case ACTION_MOVE_RIGHT:
-      if (event_type == KEY_PRESSED)
-	GoRight();
-      else
-	if (event_type == KEY_RELEASED)
-	  StopRight();
+    case Action::ACTION_MOVE_RIGHT:
+      if (event_type == Keyboard::KEY_PRESSED)
+        GoRight();
+      else if (event_type == Keyboard::KEY_RELEASED)
+        StopRight();
       break ;
 
-    case ACTION_SHOOT:
-      if (event_type == KEY_PRESSED)
-	p_Deselect();
+    case Action::ACTION_SHOOT:
+      if (event_type == Keyboard::KEY_PRESSED)
+        ActionHandler::GetInstance()->NewAction(new Action(Action::ACTION_WEAPON_STOP_USE));
       break ;
 
     default:
@@ -213,7 +217,7 @@ void JetPack::HandleKeyEvent(int action, int event_type)
 bool JetPack::p_Shoot()
 {
   m_is_active = true;
-  ActiveCharacter().SetSkin("jetpack");
+  ActiveCharacter().SetClothe("jetpack-fire");
 
   return true;
 }
@@ -223,3 +227,7 @@ void JetPack::SignalTurnEnd()
   p_Deselect();
 }
 
+void JetPack::ActionStopUse()
+{
+  p_Deselect();
+}
