@@ -1,5 +1,5 @@
 /******************************************************************************
- *  Wormux, a free clone of the game Worms from Team17.
+ *  Wormux is a convivial mass murder game.
  *  Copyright (C) 2001-2004 Lawrence Azzoug.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -27,13 +27,12 @@
 
 class SkipTurn : public Weapon
 {
- private:
-  bool p_Shoot();
-
-public:
-  SkipTurn();
-  void Refresh();
+  public:
+    SkipTurn();
+  protected:
+    bool p_Shoot();
+    void Refresh();
 };
 
 //-----------------------------------------------------------------------------
-#endif
+#endif /* SKIP_TURN_H */
