@@ -1,5 +1,5 @@
 /******************************************************************************
- *  Wormux, a free clone of the game Worms from Team17.
+ *  Wormux is a convivial mass murder game.
  *  Copyright (C) 2001-2004 Lawrence Azzoug.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -27,17 +27,16 @@
 
 class Suicide : public Weapon
 {
-private:
-   int sound_channel;
-   bool is_dying;
-
-  bool p_Shoot();
-
-public:
-  Suicide();
-  void p_Select();
-  void Refresh();
-  ExplosiveWeaponConfig &cfg();
+  private:
+    int sound_channel;
+    bool is_dying;
+  protected:
+    bool p_Shoot();
+    void p_Select();
+    void Refresh();
+  public:
+    Suicide();
+    ExplosiveWeaponConfig &cfg();
 };
 
-#endif
+#endif /* SUICIDE_H */
