@@ -1,5 +1,5 @@
 /******************************************************************************
- *  Wormux, a free clone of the game Worms from Team17.
+ *  Wormux is a convivial mass murder game.
  *  Copyright (C) 2001-2004 Lawrence Azzoug.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -16,16 +16,18 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  ******************************************************************************
- * Arme Supertux : and now the flying magic pinguin !
+ * Arme Supertux : Look ! it's the famous flying magic pinguin !
  *****************************************************************************/
 
 #include "supertux.h"
-#include "weapon_tools.h"
+#include "explosion.h"
 #include "../game/config.h"
 #include "../game/time.h"
 #include "../graphic/video.h"
+#include "../include/action_handler.h"
 #include "../interface/game_msg.h"
 #include "../map/camera.h"
+#include "../network/network.h"
 #include "../object/objects_list.h"
 #include "../team/teams_list.h"
 #include "../tool/math_tools.h"
@@ -33,64 +35,58 @@
 const uint time_delta = 40;
 const uint animation_deltat = 50;
 
-SuperTux::SuperTux(SuperTuxWeaponConfig& cfg) :
-  WeaponProjectile ("supertux", cfg), 
+SuperTux::SuperTux(SuperTuxWeaponConfig& cfg,
+                   WeaponLauncher * p_launcher) :
+  WeaponProjectile ("supertux", cfg, p_launcher),
   particle_engine(40)
 {
   explode_colliding_character = true;
+  SetSize(image->GetSize());
+  SetTestRect(1, 1, 2, 2);
 }
 
 void SuperTux::Shoot(double strength)
 {
-  Ready();
-  is_active = true;
+  WeaponProjectile::Shoot(strength);
+  angle_rad = ActiveCharacter().GetFiringAngle();
 
-  // Set physics constants.
-  ResetConstants();
-
-  // Set the initial position.
-  SetXY( ActiveCharacter().GetHandPosition() );
-
-  // Fixe la force de départ
-  angle = ActiveTeam().crosshair.GetAngleRad();
-  PutOutOfGround(angle);
-  SetExternForce(static_cast<SuperTuxWeaponConfig&>(cfg).speed, angle);
-  
   Time * global_time = Time::GetInstance();
   time_next_action = global_time->Read();
   last_move = global_time->Read();
-
-  begin_time = global_time->Read();  
-
-  ShootSound();
-
-  lst_objects.AddObject (this);
-  camera.ChangeObjSuivi(this,true,true,true);
+  begin_time = global_time->Read();
 }
 
 void SuperTux::Refresh()
 {
   WeaponProjectile::Refresh();
 
-  image->SetRotation_deg((angle+M_PI_2)*180.0/M_PI);
+  image->SetRotation_rad(angle_rad + M_PI_2);
   if ((last_move+animation_deltat)<Time::GetInstance()->Read())
-    {
-      SetExternForce(static_cast<SuperTuxWeaponConfig&>(cfg).speed, angle);
-      image->Update();
-      last_move = Time::GetInstance()->Read();
+  {
+    SetExternForce(static_cast<SuperTuxWeaponConfig&>(cfg).speed, angle_rad);
+    image->Update();
+    last_move = Time::GetInstance()->Read();
   }
 
-  particle_engine.AddPeriodic(GetPosition(), particle_STAR, false, angle, 0);
+  if(ActiveTeam().IsLocal() || ActiveTeam().IsLocalAI())
+  {
+    Action a(Action::ACTION_SUPERTUX_STATE);
+    a.Push(angle_rad);
+    a.Push(GetPhysX());
+    a.Push(GetPhysY());
+    Point2d speed;
+    network.SendAction(&a);
+  }
+  particle_engine.AddPeriodic(GetPosition(), particle_STAR, false, angle_rad, 0);
 }
 
-
 void SuperTux::turn_left()
-{  
+{
   time_now = Time::GetInstance()->Read();
   if (time_next_action<time_now)
     {
       time_next_action=time_now + time_delta;
-      angle = angle - 15.0/180.0*M_PI;
+      angle_rad = angle_rad - M_PI / 12;
     }
 }
 
@@ -100,17 +96,14 @@ void SuperTux::turn_right()
   if (time_next_action<time_now)
     {
       time_next_action=time_now + time_delta;
-      angle = angle + 15.0/180.0*M_PI;
+      angle_rad = angle_rad + M_PI / 12;
     }
 }
 
-void SuperTux::SignalCollision()
-{ 
-  if (IsGhost())
-  {
-    GameMessages::GetInstance()->Add (_("Bye bye tux..."));
-  }
-  is_active = false; 
+void SuperTux::SignalOutOfMap()
+{
+  GameMessages::GetInstance()->Add (_("Bye bye tux..."));
+  WeaponProjectile::SignalOutOfMap();
 }
 
 //-----------------------------------------------------------------------------
@@ -118,43 +111,56 @@ void SuperTux::SignalCollision()
 SuperTuxWeaponConfig::SuperTuxWeaponConfig()
 {
   speed = 2;
-  timeout = 20;
 }
 
-void SuperTuxWeaponConfig::LoadXml(xmlpp::Element *elem) 
+void SuperTuxWeaponConfig::LoadXml(xmlpp::Element *elem)
 {
-  WeaponConfig::LoadXml (elem);
-  LitDocXml::LitUint (elem, "speed", speed);
+  ExplosiveWeaponConfig::LoadXml (elem);
+  XmlReader::ReadUint(elem, "speed", speed);
 }
 
 //-----------------------------------------------------------------------------
 
-TuxLauncher::TuxLauncher() : 
+TuxLauncher::TuxLauncher() :
   WeaponLauncher(WEAPON_SUPERTUX, "tux", new SuperTuxWeaponConfig(), VISIBLE_ONLY_WHEN_INACTIVE)
-{ 
-  m_name = _("SuperTux");   
+{
+  m_name = _("SuperTux");
   override_keys = true ;
-
-  projectile = new SuperTux(cfg());
+  ReloadLauncher();
 }
 
-void TuxLauncher::HandleKeyEvent(int action, int event_type)
+WeaponProjectile * TuxLauncher::GetProjectileInstance()
 {
-  switch (action) {
-  case ACTION_MOVE_LEFT:
-    if (event_type != KEY_RELEASED)
-      static_cast<SuperTux *>(projectile)->turn_left();
+  return dynamic_cast<WeaponProjectile *>
+      (new SuperTux(cfg(),dynamic_cast<WeaponLauncher *>(this)));
+}
+
+bool TuxLauncher::p_Shoot ()
+{
+  current_tux = static_cast<SuperTux *>(projectile);
+  return WeaponLauncher::p_Shoot();
+}
+
+void TuxLauncher::HandleKeyEvent(Action::Action_t action, Keyboard::Key_Event_t event_type)
+{
+  switch (action)
+  {
+    case Action::ACTION_MOVE_LEFT:
+    if (event_type !=  Keyboard:: Keyboard::KEY_RELEASED)
+      current_tux->turn_left();
     break ;
-    
-  case ACTION_MOVE_RIGHT:
-    if (event_type != KEY_RELEASED)
-      static_cast<SuperTux *>(projectile)->turn_right();
+
+    case Action::ACTION_MOVE_RIGHT:
+    if (event_type !=  Keyboard:: Keyboard::KEY_RELEASED)
+      current_tux->turn_right();
     break ;
-    
+
   default:
     break ;
   } ;
 }
 
-SuperTuxWeaponConfig& TuxLauncher::cfg() 
-{ return static_cast<SuperTuxWeaponConfig&>(*extra_params); }
+SuperTuxWeaponConfig& TuxLauncher::cfg()
+{
+  return static_cast<SuperTuxWeaponConfig&>(*extra_params);
+}
